/*
 **************
 * Quest demo *
 **************

 This is the general game code.
 It's just stuff to make the game work; you'll learn the most about
 the library by looking at the quest_ files where quests are implemented.

 To keep from reinventing the wheel, this demo uses several libraries:
 	- Spuzzum's s_eye library keeps the camera from leaving the map.
 	- AbyssDragon's tile libraries are used for the icons.
*/


// The demo makes these additions and modifications to the Quest class.
BaseCamp/quest
	QuestRejected()
		demo_close_browser(player)
		return ..()

	PlayerKilledMob(mob/victim)
		var/atom/item = ..()
		if (item)
			player << "You receive the [item.name]."
		return item

	proc/ChooseRandomNPC(mob/quest_giver)
		// Choose a random NPC who is not the quest giver.
		// Often used to choose next step of the quest.
		var/mob/npc/rand_npc
		while (!rand_npc || rand_npc == quest_giver)
			rand_npc = pick(npc_list)
		return rand_npc

var/global/list/npc_list

mob/npc
	/*
	   We use the mob/npc type to represent non-monster NPCs in the game.
	   For quest convenience, a global list of NPCs is kept so NPCs can be easily chosen/found.
	*/
	gender = MALE

	New()
		if (!npc_list)
			npc_list = new()
		npc_list += src
		return ..()

	Del()
		npc_list -= src
		return ..()

	Click()
		// Are they close enough to talk to me?
		if (get_dist(src, usr) > 2)
			return

		// Do I have a quest-related action for this player?
		if (base_QuestAction(usr))
			return

mob/npc/barney
	name = "Bearded Barney"
	icon = 'Sage.dmi'

mob/npc/lucky
	name = "Happy Go Lucky"
	icon = 'Shopkeeper.dmi'

mob/npc/fred
	name = "Flamboyent Fred"
	icon = 'good knight.dmi'

mob/npc/suzie
	name = "Suzie Saphron"
	icon = 'Barmaid.dmi'
	gender = FEMALE


world
	view = 8

mob
	Login()
		// Set icon based on gender.
		if (gender == FEMALE)
			icon = 'Valkrye.dmi'
		else
			icon = 'Rogue.dmi'

		/*
		   Place player in center of map.
		   Just set their loc instead of using Move, because
		   we want to guarantee that it succeeds.
		 */
		var/center_x = round(world.maxx / 2)
		var/center_y = round(world.maxy / 2)
		var/turf/center = locate(center_x, center_y, 1)
		loc = center

		// Display the intro message to the player.
		spawn()
			src << browse('introduction.html', "window=intro;size=600x400")
		return ..()

	Stat()
		if(src == usr)
			statpanel("inventory", contents)

	base_PlayerKilledMob(mob/victim)
		src << "After an extended battle, you killed the [victim]!"
		return ..()

	proc/Die(mob/killer)
		del(src)

turf/ground
	/*
	   Ground icons use random icon states if an icon state hasn't been set.
	   This means the map looks different each time it's loaded.
	 */
	icon = 'landscape.dmi'

	var/global/list/states

	New()
		if (!icon_state)
			if (!states)
				states = icon_states(icon)
			icon_state = pick(states)
		return ..()

turf/ground/dirt
	icon_state = "dirt"

proc/rpg_browse(player, message)
	// This provides a consistent window/background color/font to pop-up messages.
	var/page = {"<html><body bgcolor="black" text="#00ff00"><font size="+1">[message]</font></body></html>"}
	player << browse(page, "window=browse;size=500x400")

proc/demo_close_browser(player)
	player << browse(null, "window=browse")

proc/direction(source, dest)
	var/direction = get_dir(source, dest)
	switch(direction)
		if (NORTH) return "North"
		if (WEST)  return "West"
		if (EAST)  return "East"
		if (SOUTH) return "South"
		if (NORTHWEST) return "Northwest"
		if (NORTHEAST) return "Northeast"
		if (SOUTHWEST) return "Southwest"
		if (SOUTHEAST) return "Southeast"


// Blubbers respawn in their area when killed.
area
	proc/SpawnMobDied(mob/dead)
		// Respawn a mob of this type in random time in a random place in the area.
		var/mob_type = dead.type

		// Spawn another mob in next 2 minutes, waiting at least 10 seconds.
		spawn(rand(1100) + 100)
			// Choose a random spot from my corner.
			var/turf/corner = locate(type)

			var/random_x
			var/random_y
			var/turf/random_turf

			// Keep looking until we find a random spot in my area.
			do
				random_x = corner.x + rand(10)
				random_y = corner.y + rand(10)
				random_turf = locate(random_x, random_y, z)
			while (random_turf.loc != src)

			new mob_type(random_turf)


area/blubber

obj/wall
	name = "Wall"
	icon = 'wall.dmi'
	icon_state = "egyptian"
	density = 1

mob/monster
	Click()
		// Are they close enough to fight me?
		if (get_dist(src, usr) > 1)
			return

		Die(usr)
		return

	Die(mob/killer)
		// If a player killed me, let them know.
		if (killer && killer.client)
			killer.base_PlayerKilledMob(src)
		return ..()


mob/monster/blubber
	name = "Blubber"
	icon = 'Slime.dmi'

	var/tmp
		area/spawn_area

	New()
		var/result = ..()
		spawn_area = dd_area()
		return result

	Die()
		spawn_area.SpawnMobDied(src)
		return ..()

mob/monster/ogre
	name = "Ogre"
	icon = 'Ogre.dmi'

mob/monster/statue
	name = "Menacing Statue"
	icon = 'statue.dmi'
	icon_state = "right"