/*
 *************************
 * Blubber Bladder quest *
 *************************

 The simplest form of quest: Kill 3 blubbers and return their
 bladders to the quest giver.

 Two unique things about this quest:

 1) It is randomly assigned to an NPC. If you are eligible for the quest,
    there is a 50% chance that any NPC will offer it to you.

 2) Using the quest functionality, if you attempt to talk to a different
    NPC while carrying a bladder, they will refuse to interact with you
    because of the smell. This demonstrates how powerful the engine is:
    literally every NPC in your game can be involved in a quest, without
    having any quest-specific code programmed into them.

 This quest uses the default functionality for getting special loot by killing a monster.
 To do this, it specifies the victim_type and loot_type, causing a bladder to
 be given to the player each time they kill a Blubber.
*/

mob/npc
	New()
		if (!base_quest_types_available)
			base_quest_types_available = new()
		base_quest_types_available += /BaseCamp/quest/blubber
		return ..()

BaseCamp/quest/blubber
	name = "Blubber Bladders Quest"
	victim_type = /mob/monster/blubber
	loot_type = /obj/blubber_bladder
	loot_count = 3

	isAvailableForPlayer(mob/player)
		// 50% chance that it's available if they don't already have a Blubber quest
		if (!player.base_hasQuestOfType(type) && prob(50))
			return 1
		return 0

	InitiateQuestDialog(atom/quest_giver)
		// Fireworks Sorcerer gives the quest.
		var/blubber_bladders = {"<a href="?command=offer_quest;src=\ref[src];npc=\ref[quest_giver]">Blubber bladders</a>"}
		rpg_browse(player, "I am in serious need of some [blubber_bladders].")

	OfferQuest(atom/quest_giver)
		rpg_browse(player, "Don't ask me why, but I could really use some.")
		return ..()

	QuestAction(atom/npc)
		/*
		   The NPC can be anyone, so we have to figure out if it's someone related to this quest.
		*/
		var/count = LootCount()

		// If the NPC isn't the quest giver, then see if they are repulsed by a bladder.
		if (!isQuestGiver(npc))
			if (count)
				var/message = pick("Pee-yoo, you reek! Get out of my sight until you've gotten rid of the source of that smell!",
				                   "Have you no decency? I swear, when I was a kid we didn't go around waving bladders under people's noses!",
				                   "That thing in your backpack is disgusting! Go away!")
				rpg_browse(player, message)
				return 1

			// Nothing to do.
			return 0

		// The NPC is the quest giver, so respond to the player depending on how much of the quest they've completed.
		if (!count)
			// They don't have any bladders yet!
			// Locate a Blubber and point them in that direction.
			var/mob/monster/blubber/a_blubber = locate()

			if (a_blubber)
				var/direction = direction(npc, a_blubber)
				rpg_browse(player, "Please go kill [loot_count] Blubbers for me and return their bladders. The disgusting things tend to hang out to the [direction].")
			else
				rpg_browse(player, "Please kill [loot_count] Blubbers for me and return their bladders. Unfortunately, there don't seem to be any around at the moment, but they always ooze out sooner or later.")
		else if (count >= loot_count)
			// They got all the bladders.
			DeleteLoot()
			rpg_browse(player, "Wonderful! You have the bladders, now I will be able to...never mind, just thanks for getting them.")
			QuestCompleted()
		else
			// They don't have enough.
			rpg_browse(player, "Sorry, but I can't accept a partial delivery. You need all [loot_count] Blubber bladders.")
		return 1

obj/blubber_bladder
	name = "Blubber Bladder"
	icon = 'small slime.dmi'