/*
 **********************
 * Dirty Dagger quest *
 **********************

 This is a bit more complicated quest.  It supports subquests, assigned in random order.
 (Only one subquest is implemented here though).

 In the subquest, when you try to get an item from an NPC, the NPC creates two
 monsters to attack you.

 When you've done the subquest, the Smith will teleport you to a maze for the final part
 of the quest.

*/


mob/npc/smith
	name = "Small Weapon Smith"
	icon = 'old man.dmi'

	New()
		var/result = ..()
		if (!base_quest_types_available)
			base_quest_types_available = new()
		base_quest_types_available += /BaseCamp/quest/dagger
		return result

BaseCamp/quest/dagger
	name = "Dirty Dagger Quest"
	multiple_times_allowed = 0

	// For the blade part of the quest, killing the statue gives the blade.
	victim_type = /mob/monster/statue
	loot_type = /obj/component/dagger/blade

	var
		BaseCamp/quest/dagger/current_subquest
		list/subquest_types_remaining

	New()
		subquest_types_remaining = typesof(/BaseCamp/quest/dagger) - /BaseCamp/quest/dagger
		return ..()

	InitiateQuestDialog(atom/quest_giver)
		var/Dirty_Daggers = {"<a href="?command=offer_quest;src=\ref[src];npc=\ref[quest_giver]">Dirty Daggers</a>"}
		rpg_browse(player, "No money, eh? Well it's not much, but perhaps I can set you up with one of my special [Dirty_Daggers].")

	OfferQuest(atom/quest_giver)
		rpg_browse(player, "The extra bite comes from the rust, you know. Anyway, you'll need to do some work to get it...")
		return ..()

	QuestAction(atom/npc)
		// Is this the original quest giver?
		if (isQuestGiver(npc))
			if (!current_subquest)
				var/components = DaggerComponentCount()
				if (!components)
					// We're starting the quest.
					var/mob/npc/subquest_npc = SelectSubquest(npc)
					var/direction = direction(npc, subquest_npc)
					rpg_browse(player, "You can't get something for nothing, kid...you'll need to do the work to get the dagger parts, then I'll put it together.<p>Let's get started here...hmm, go see [subquest_npc]. \He hangs out to the [direction].")
					return 1
				else
					if (components == 1)
						// They have the dagger handle, teleport them to the maze.
						rpg_browse(player, "Now we're getting somewhere, kid. This last half will be a bit tougher.<p>Hang on to your britches, you are taking a trip!")
						TeleportPlayerToMaze()
						return 1
					else
						// They have more than one component, so they have all the parts.
						for (var/obj/component/dagger/item in player.contents)
							del(item)
						new /obj/dagger(player)
						rpg_browse(player, "Well kid you did it! Here's the dirty dagger. Don't try cutting your toenails with it.")
						QuestCompleted()
						return 1
			else
				// They have a subquest and haven't completed it...
				rpg_browse(player, "Oh, hi! How's the whole dagger thing going?")
				return 1

		// If there is no subquest or this isn't the subquest giver, then nothing to do.
		if (!current_subquest || !current_subquest.isQuestGiver(npc))
			return 0

		// This is the current subquest giver, so pass it on to the subquest.
		return current_subquest.QuestAction(npc)

	proc/DaggerComponentCount()
		var/count = 0
		for (var/obj/component/dagger/item in player.contents)
			count++
		return count

	proc/SelectSubquest(atom/quest_giver)
		// Pick an NPC who will be the quest giver for this part of the quest.
		var/mob/npc/quest_npc = ChooseRandomNPC(quest_giver)

		// Create the quest.
		var/subquest_type = pick(subquest_types_remaining)
		subquest_types_remaining -= subquest_type

		current_subquest = new subquest_type()
		current_subquest.Start(player, quest_npc)
		return quest_npc

	proc/TeleportPlayerToMaze()
		TeleportPlayer(locate("dagger_teleport"))

	proc/TeleportPlayerToSmith()
		// Teleport the player to the maze, causing the player to flash first.
		var/mob/smith = locate(/mob/npc/smith)
		TeleportPlayer(player.LocationNear(smith))

	proc/TeleportPlayer(turf/destination)
		var/player_icon = player.icon
		for (var/count = 6; count > 0; count--)
			if (player.icon)
				player.icon = null
			else
				player.icon = player_icon
			sleep(5)

		player.loc = destination
		player.icon = player.icon

	PlayerKilledMob(mob/victim)
		// If we have a subquest, let it know.
		if (current_subquest)
			return current_subquest.PlayerKilledMob(victim)

		// Otherwise if it's the statue this is the end of the quest.
		if (victim.type == victim_type)
			// Let default behavior give the player the blade.
			..()

			// Teleport them back.
			TeleportPlayerToSmith()
			return 1
		return 0


BaseCamp/quest/dagger/attack
	/*
	   To avoid giving up the loot, this NPC sics attackers on the player.
	   When the attackers have been killed, the NPC gives in.
	*/
	var/tmp
		attack_started = 0
		list/attackers = new()

	QuestAction(atom/npc)
		// If we haven't started the attack yet, set them up for it...
		if (!attack_started)
			var/just_go_away = {"<a href="?command=go_away;src=\ref[src];npc=\ref[npc]">just go away</a>"}
			rpg_browse(player, "I guess you're here for the dagger handle I told the smith I owed him someday, eh?<p>Well, um, hmm, things have come up, and, well, why don't you [just_go_away].")
			return 1
		else
			// If the attackers aren't dead...
			if (attackers.len)
				rpg_browse(player, "Heh heh, my minions will shred you!")
				return 1
			else
				rpg_browse(player, "Oops, heh heh. You are, uh, a bit stronger than I was led to believe.<p>How's about I give you that dagger handle now and you don't mention anything to the smith, alrighty?")
				var/obj/component/dagger/handle/item = new(player)
				player << "You receive the [item.name]."

				// Delete subquest after indicating success.
				spawn()
					del(src)
				return 1

	Topic(href, href_list[])
		var/command = href_list["command"]

		switch(command)
			if ("go_away")
				StartAttack()
				return
		return ..()

	proc/StartAttack()
		rpg_browse(player, "I warned you! I told you to go away, but you didn't listen!<p>Get him, boys!")

		for(var/num_monsters = 1, num_monsters <= 2, num_monsters++)
			var/mob/monster/ogre/attacker = new()

			// Find a random spot the monster can appear at.
			attacker.Move(attacker.LocationNear(player))
			attackers += attacker

		attack_started = 1

	PlayerKilledMob(mob/victim)
		if (victim in attackers)
			attackers -= victim
			return 1
		return 0

obj/component/dagger/handle
	name = "dagger handle"
	icon = 'torch.dmi'

obj/component/dagger/blade
	name = "dagger blade"
	icon = 'lantern.dmi'

obj/dagger
	name = "Dirty Dagger"
	icon = 'sword.dmi'

mob/proc/LocationNear(atom/O)
	// Find a random spot near the destination and move there.
	var/random_x = O.x
	var/random_y = O.y
	var/turf/random_turf = locate(O.x, O.y, 1)
	while (!random_turf.Enter(src))
		random_x = min(world.maxx, random_x + rand(2))
		random_y = min(world.maxy, random_y + rand(2))
		random_turf = locate(random_x, random_y, 1)
	return random_turf

/***
obj/test/verb/dagger_attack_test()
	var/mob/test_player = new(locate(5,5,1))

	var/BaseCamp/quest/dagger/attack/test_quest = new()
	test_quest.player = test_player

	test_quest.StartAttack()
	if (!test_quest.attackers.len)
		die("[test_quest.type].StartAttack() did not create monsters.")

	var/mob/thing
	for (thing in test_quest.attackers)
		if (!thing.loc)
			die("[test_quest.type].StartAttack() did not place a monster on the map.")

	test_quest.PlayerKilledMob(thing)
	if (test_quest.attackers.Find(thing))
		die("[test_quest.type].PlayerKilledMob() didn't remove mob from list.")

	del(test_player)
	for (var/item in test_quest.attackers)
		del(item)
***/