/*
 *******************
 * Fireworks quest *
 *******************

 A typical FedEx quest, but with a twist. You deliver package to
 X and return to the explosives expert with what X gives you.
 X is chosen at random, showing how you can make your quests
 very dynamic. The twist is this: When the quest has been done 5
 times, the fireworks expert puts on a fireworks display.
*/

mob/npc/sorceror
	name = "Fireworks Sorceror"
	icon = 'sorceror.dmi'

	New()
		var/result = ..()
		if (!base_quest_types_available)
			base_quest_types_available = new()
		base_quest_types_available += /BaseCamp/quest/fireworks
		return result

BaseCamp/quest/fireworks
	name = "Fireworks Quest"
	loot_type = /obj/firework_component

	var
		errand_mob_type			// The type of the mob the player needs to get something from. Could also use tag.
		errands_remaining = 4	// Errands left to do fireworks display.
		firework_component_type	// What they are getting from errand mob.

	var/tmp
		mob/errand_mob

//	isAvailableForPlayer(mob/player)
//		return 1

	InitiateQuestDialog(atom/quest_giver)
		// Fireworks Sorcerer gives the quest.
		var/here_about_fireworks = {"<a href="?command=offer_quest;src=\ref[src];npc=\ref[quest_giver]">here about fireworks</a>"}
		rpg_browse(player, "I'm a busy man. Busy busy busy. I don't suppose you are [here_about_fireworks], are you?")

	OfferQuest(atom/quest_giver)
		rpg_browse(player, "Yes, as it happens, I can make fireworks. But if you want to see fireworks you are going to have do some of my errands for me.")
		return ..()

	QuestAction(atom/npc)
		/*
		   The NPC can be anyone, so we have to figure out if it's someone related to this quest.
		   Could be the quest giver, could be an errand destination, could be someone unrelated.
		*/
		var/is_quest_giver
		var/is_errand_mob

		if (isQuestGiver(npc))
			is_quest_giver = 1
		else if (npc.type == errand_mob_type)
			is_errand_mob = 1

		// If the NPC isn't the quest giver or the errand mob, then nothing to do here.
		if (!is_quest_giver && !is_errand_mob)
			return 0

		if (is_quest_giver)
			if (!errand_mob_type)
				ChooseErrandMob(npc)
				var/direction = direction(npc, errand_mob)
				rpg_browse(player, "You have [errands_remaining] errand\s to do for me. Go to [errand_mob] and bring what \he gives you back to me. Look to the [direction].")
				return 1
			else
				/*
				   They were already given an errand.
				   Do they have the firework component in their contents?
				*/
				if (LootCount())
					// They have the item -- delete it then check for more errands.
					DeleteLoot()

					errands_remaining--
					if (errands_remaining)
						ChooseErrandMob(npc)
						var/direction = direction(npc, errand_mob)
						rpg_browse(player, "Thank you. This is just what I needed. But you have [errands_remaining] more thing\s to do! Now go to [errand_mob] and bring me what \he gives you. You'll find \him in the [direction].")
						return 1
					else
						rpg_browse(player, "Thank you for all your help. An old man needs all he can get. Now I will put on a fireworks show for you!")

						spawn(30)
							FireworksShow(npc)
							QuestCompleted()
						return 1

				// They don't have the quest item yet.
				rpg_browse(player, "If you don't have what I asked for, I really don't have time to talk. Busy busy.")
				return 1

		// This is the errand mob, so give the player the item if they don't have it already.
		if (LootCount())
			// They have it, so nothing to do for them.
			rpg_browse(player, "How are those deliveries going?")
		else
			var/obj/firework_component/item = new(player)
			rpg_browse(player, "Doing errands for the old coot, eh? Well, here is the [item.name] he wants!")
			player << "You receive the [item.name]."
		return 1

	proc/ChooseErrandMob(mob/npc)
		errand_mob = ChooseRandomNPC(npc)
		errand_mob_type = errand_mob.type

	proc/FireworksShow(mob/npc)
		// In a block around the wizard, flick the fireworks.
		var/upper_x = max(npc.x - 3, 1)
		var/upper_y = min(npc.y + 3, world.maxy)
		var/upper_turf = locate(upper_x, upper_y, 1)

		var/lower_x = min(npc.x + 3, world.maxx)
		var/lower_y = max(npc.y - 3, 1)
		var/lower_turf = locate(lower_x, lower_y, 1)

		for(var/turf/T in block(upper_turf, lower_turf))
			flick('firework.dmi', T)

		world << "Bang! Boom! Pretty colors!"

obj/firework_component
	New()
		name = pick("sulfur", "wrapping paper", "box of fuses", "clean underwear")
		return ..()